import sys, os, curses, logging.config, threading
from time import sleep
from .interfaces import TextAccumulator, OptionWindow

from random import randint
logging.config.dictConfig({
    'version': 1,
    'disable_existing_loggers': True,
})

highlighted_slot_index = 0
option_window = OptionWindow()


global_text_accumulator = TextAccumulator()

def input_thread(stdscr):
    global current_slot_index
    global option_window
    curses.curs_set(0)
    k = 0
    
    while (k != ord('q')):
        highlighted_slot_index = option_window.highlight_ticker
        if k == curses.KEY_DOWN:
            option_window.remove_focus()
            highlighted_slot_index = (highlighted_slot_index + 2) % 16
            option_window.focus_option_at_key(highlighted_slot_index)
        if k == curses.KEY_UP:
            option_window.remove_focus()
            highlighted_slot_index = (highlighted_slot_index - 2) % 16
            option_window.focus_option_at_key(highlighted_slot_index)
        if k == curses.KEY_RIGHT:
            if option_window.options.get(highlighted_slot_index).text != "LOADING...":
                current_text = option_window.options.get(highlighted_slot_index).text
                global_text_accumulator.add_text(current_text)
                option_window.set_screen(stdscr)
                current_slot_index = 0
                option_window.highlight_ticker = 0
        if k == curses.KEY_LEFT:
            global_text_accumulator.backspace()
            option_window.set_screen(stdscr)
            current_slot_index = 0
            option_window.highlight_ticker = 0

    
        for box in option_window.options.values():
            box.refresh()
        stdscr.refresh()
        # Wait for next input
        k = stdscr.getch()
    

def draw_menu(stdscrr):
    stdscr = curses.initscr()
    height, width = stdscr.getmaxyx()
    # Start colors in curses
    stdscr.refresh()
    curses.start_color()

    new_box = curses.newwin(1, width, height-2, 0)
    new_box.attron(curses.color_pair(1))
    new_box.attron(curses.A_BOLD)
    new_box.addnstr(0, 0, "UP/DOWN Selects text. RIGHT Adds selected text block to game. LEFT Removes rightmost character from the game. q exits.", width-1)
    new_box.attroff(curses.color_pair(1))
    new_box.attroff(curses.A_BOLD)
    new_box.refresh()



    curses.init_pair(1, curses.COLOR_CYAN, curses.COLOR_BLACK)
    curses.init_pair(2, curses.COLOR_RED, curses.COLOR_BLACK)
    curses.init_pair(3, curses.COLOR_BLACK, curses.COLOR_WHITE)
    option_window.set_screen(stdscr)
    global_text_accumulator.set_screen(stdscr)

    t = threading.Thread(name ='daemon', target=input_thread, args=(stdscr,))
    t.setDaemon(True)
    t.start()

    while t.is_alive():
        option_window.attempt_to_generate_new_option_at_text_ticker_location(global_text_accumulator.generate_add_text_candidate())
        sleep(1)

        option_window.focus_option_at_key(option_window.highlight_ticker)
        for box in option_window.options.values():
            box.refresh()
        stdscr.refresh()

def main():
    curses.wrapper(draw_menu)

if __name__ == "__main__":
    main()



from transformers import pipeline
from transformers import GPT2LMHeadModel,  GPT2Tokenizer, GPT2Config, GPT2LMHeadModel
import torch
import os
from transformers import AutoTokenizer, AutoModelWithLMHead

class TextGenerator():
	def __init__(self):
		if os.path.isdir("model/"):
			print("using local model")
			self.model = GPT2LMHeadModel.from_pretrained('model/')
			self.tokenizer = GPT2Tokenizer.from_pretrained('model/')
		else:
			print("using gyre/200wordrpgmodel model")
			self.model = AutoModelWithLMHead.from_pretrained("gyre/200wordrpgmodel")
			self.tokenizer = AutoTokenizer.from_pretrained("gyre/200wordrpgmodel")

		self.device = torch.device("cpu")
		self.model.to(self.device)
		self.model.eval()
    
	def create_textblock(self, text_on_screen=""):
		prompt="<|startoftext|>" + text_on_screen
		raw_prompt_length = len(text_on_screen)
		prompt_length = len(prompt.split(' '))
		generated = torch.tensor(self.tokenizer.encode(prompt)).unsqueeze(0)
		generated = generated.to(self.device)
		sample_outputs = self.model.generate(
										generated, 
										do_sample=True,   
										top_k=50, 
										max_length = list(generated.size())[1] + 10,
										top_p=0.95, 
										num_return_sequences=1
										)
		new_block = self.tokenizer.decode(sample_outputs[0], skip_special_tokens=True)
		return new_block[raw_prompt_length:]


from setuptools import setup, find_packages
setup(name='transformstorm',
      version='0.1.5',
      packages = find_packages(),
      description='A command line interface for playing with language models',
      url='https://github.com/simonmnash/transformstorm',
      author='Simon Nash',
      license='MIT',
      install_requires=['transformers', 'torch'],
      scripts=["bin/transformstorm"]
      )


import curses, os, textwrap
from .textgenerator import TextGenerator



class TextAccumulator():
    def __init__(self):
        self.complete_text = ""
        self.text_generator = TextGenerator()
        
    def set_screen(self, stdscr):
        self.screen = stdscr

    def add_text(self, text):
        height, width = self.screen.getmaxyx()
        self.complete_text = self.complete_text + text
        new_box = curses.newwin(height-5, 60, 0, 75)
        new_box.attron(curses.color_pair(2))
        new_box.attron(curses.A_BOLD)
        new_box.addnstr(0, 0, textwrap.fill(self.complete_text, 100), width)
        new_box.attroff(curses.color_pair(2))
        new_box.attroff(curses.A_BOLD)
        new_box.refresh()
    def backspace(self):
        height, width = self.screen.getmaxyx()
        self.complete_text = self.complete_text[:-1]
        new_box = curses.newwin(height-5, 60, 0, 75)
        new_box.attron(curses.color_pair(2))
        new_box.attron(curses.A_BOLD)
        new_box.addnstr(0, 0, textwrap.fill(self.complete_text, 100), width)
        new_box.attroff(curses.color_pair(2))
        new_box.attroff(curses.A_BOLD)
        new_box.refresh()
    def generate_add_text_candidate(self):
        return self.text_generator.create_textblock(self.complete_text)


class TextBox():
    def __init__(self, slot, text, highlighted = False):
        self.text = text
        self.slot = slot
        self.highlighted = highlighted
        self.box = self.draw()
    
    def draw(self):
        new_box = curses.newwin(2, 75, self.slot, 0)
        if self.highlighted:
            new_box.attron(curses.color_pair(3))
            new_box.attron(curses.A_BOLD)
        new_box.addstr(1, 2, textwrap.fill(self.text, 75))
        if self.highlighted:
            new_box.attroff(curses.color_pair(3))
            new_box.attroff(curses.A_BOLD)
        return new_box

    def focus(self):
        self.highlighted = True
        self.box = self.draw()
        return self
    
    def unfocus(self):
        self.highlighted = False
        self.box = self.draw()
        return self
    
    def toggle_highlight(self):
        if self.highlighted:
            self.highlighted = False
        else:
            self.highlighted = True
        self.box = self.draw()
        self.box.refresh()

    def refresh(self):
        self.box.refresh()

class OptionWindow():
    def __init__(self):
        self.options = {}
        # highlight ticker determines which option is currently highlighted.
        self.highlight_ticker = 0
        # text generation ticker determines which option is next in line for replacement.
        self.text_generation_ticker = 0

    def set_screen(self, stdscr):
        self.screen = stdscr
        height, width = stdscr.getmaxyx()
        for slot in range(0, 12, 2):
            self.add_new_option(slot, "LOADING...", slot==0)

    def add_new_option(self, new_option_location, new_option_text, highlighted):
        new_text_box = TextBox(new_option_location, new_option_text, highlighted)
        self.options[new_option_location] = new_text_box

    def attempt_to_generate_new_option_at_text_ticker_location(self, new_option_text):
        if self.text_generation_ticker == self.highlight_ticker and self.options[self.text_generation_ticker].text != "LOADING...":
            pass
        else:
            self.add_new_option(self.text_generation_ticker, new_option_text, False)
        self.text_generation_ticker = (self.text_generation_ticker + 2) % 12

    def remove_focus(self):
        for option in self.options.values():
            self.options[option.slot] = option.unfocus()

    def focus_option_at_key(self, key_to_focus):
        if self.options.get(key_to_focus):
            self.highlight_ticker = key_to_focus
            self.options[key_to_focus] = self.options[key_to_focus].focus()
